package java.rmi.server;

import sun.rmi.transport.proxy.RMIMasterSocketFactory;
import java.net.SocketException;
import java.net.ServerSocket;
import java.io.IOException;
import java.net.Socket;

public abstract class RMISocketFactory implements RMIClientSocketFactory, RMIServerSocketFactory
{
    private static RMISocketFactory factory;
    private static RMISocketFactory defaultSocketFactory;
    private static RMIFailureHandler handler;
    
    public RMISocketFactory() {
    }
    
    @Override
    public abstract Socket createSocket(String p0, int p1) throws IOException;
    
    @Override
    public abstract ServerSocket createServerSocket(int p0) throws IOException;
    
    public static synchronized void setSocketFactory(RMISocketFactory factory) throws IOException {
        if (RMISocketFactory.factory != null) {
            throw new SocketException("factory already defined");
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkSetFactory();
        }
        RMISocketFactory.factory = factory;
        return;
    }
    
    public static synchronized RMISocketFactory getSocketFactory() {
        return RMISocketFactory.factory;
    }
    
    public static synchronized RMISocketFactory getDefaultSocketFactory() {
        if (RMISocketFactory.defaultSocketFactory == null) {
            RMISocketFactory.defaultSocketFactory = new RMIMasterSocketFactory();
        }
        return RMISocketFactory.defaultSocketFactory;
    }
    
    public static synchronized void setFailureHandler(RMIFailureHandler handler) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkSetFactory();
        }
        RMISocketFactory.handler = handler;
    }
    
    public static synchronized RMIFailureHandler getFailureHandler() {
        return RMISocketFactory.handler;
    }
    
    static {
        RMISocketFactory.factory = null;
        RMISocketFactory.handler = null;
    }
}
